package co.jp.springp0421.dogdemo.common;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

@Data
@AllArgsConstructor
@NoArgsConstructor
public class ApiResponse<T> {
    //成功状况判定
    private boolean success;
    //状态码
    private int code;
    //状态信息
    private String message;
    //数据
    private T data;

    public static <T> ApiResponse<T> success(T data) {

        return new ApiResponse<>(true, ResultCode.SUCCESS.getCode(), ResultCode.SUCCESS.getMessage(), data);
    }

    public static <T> ApiResponse<T> success() { // 通常也会有一个不带数据的成功响应
        return new ApiResponse<>(true, ResultCode.SUCCESS.getCode(), ResultCode.SUCCESS.getMessage(), null);
    }

    public static <T> ApiResponse<T> fail(ResultCode resultCode) {
        if (resultCode == ResultCode.SUCCESS) {
            throw new IllegalArgumentException("Cannot use SUCCESS ResultCode for fail method. Use a specific error code.");
        }
        // 修正：success 应为 false，并从 resultCode 获取 code 和 message
        return new ApiResponse<>(false, resultCode.getCode(), resultCode.getMessage(), null);
    }

    public static <T> ApiResponse<T> fail(ResultCode resultCode, T data) {
        if (resultCode == ResultCode.SUCCESS) {
            throw new IllegalArgumentException("Cannot use SUCCESS ResultCode for fail method. Use a specific error code.");
        }
        // 修正：success 应为 false，并从 resultCode 获取 code 和 message
        return new ApiResponse<>(false, resultCode.getCode(), resultCode.getMessage(), data);
    }
}
