package co.jp.springp0421.dogdemo.service;

import java.util.Collection;
import java.util.Collections;

import co.jp.springp0421.dogdemo.common.ResultCode;
import co.jp.springp0421.dogdemo.dto.RegistrationDto;
import co.jp.springp0421.dogdemo.entity.UserEntity;
import co.jp.springp0421.dogdemo.exception.BusinessException;
import co.jp.springp0421.dogdemo.repository.UserRepository;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.jetbrains.annotations.NotNull;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class UserService extends ServiceImpl<UserRepository, UserEntity> implements UserDetailsService {

	private final UserRepository userRepository;
	private final PasswordEncoder passwordEncoder;

	public UserService(UserRepository userRepository, PasswordEncoder passwordEncoder) {
		this.userRepository = userRepository;
		this.passwordEncoder = passwordEncoder;
	}

	public UserEntity registerNewUser(@NotNull RegistrationDto registrationDto) throws BusinessException {

		if (userRepository.existsByEmail(registrationDto.getEmail())) {
			throw new BusinessException(ResultCode.USER_EMAIL_ALREADY_EXISTS,"error: Email" + registrationDto.getEmail() + " had been used");
		}

		UserEntity newUser = new UserEntity();
		newUser.setName(registrationDto.getName());
		newUser.setEmail(registrationDto.getEmail());
		newUser.setPassword(passwordEncoder.encode(registrationDto.getPassword()));

		boolean success = this.save(newUser);

		if (success) {

			return newUser;
		} else {

			throw new BusinessException(ResultCode.DATABASE_ERROR, "User registration failed");
		}
	}

	@Override
	@Transactional(readOnly = true)
	public UserDetails loadUserByUsername(String email) throws UsernameNotFoundException {
		UserEntity userEntity = userRepository.findByEmail(email)
				.orElseThrow(() -> new UsernameNotFoundException(email + " not found"));

		Collection<? extends GrantedAuthority> authorities = Collections
				.singletonList(new SimpleGrantedAuthority("ROLE_USER"));

		return new User(
				userEntity.getEmail(),
				userEntity.getPassword(),
				true, // enabled
				true, // accountNonExpired
				true, // credentialsNonExpired
				true, // accountNonLocked
				authorities // role
		);
	}
}